@@ -1,4 +1,14 @@
 import os
+from functools import partial
+
+import six
+
+from itertools import chain, repeat
+
+if six.PY3:
+    izip = zip
+else:
+    from itertools import izip
 # -*- coding: utf-8 -*-
 
 """
@@ -38,23 +48,29 @@ def list_to_dataframe(rows, names):
     :params names: the column names for the dataframe
     """
     from pandas import DataFrame
+    col_convert_func = None
     try:
         import pandas._tseries as lib
+        col_convert_func = lib.convert_sql_column
     except ImportError:
         import pandas.lib as lib
+        try:
+            col_convert_func = lib.convert_sql_column
+        except:
+            col_convert_func = partial(lib.maybe_convert_objects, try_float=1)
 
     if isinstance(rows, tuple):
         rows = list(rows)
 
     columns = dict(zip(names, lib.to_object_array_tuples(rows).T))
 
-    for k, v in columns.iteritems():
-        columns[k] = lib.convert_sql_column(v)
+    if col_convert_func is not None:
+        for k, v in columns.iteritems():
+            columns[k] = col_convert_func(v)
 
     return DataFrame(columns, columns=names)
 
 
-from itertools import izip, chain, repeat
 
 def array_pagenate(n, iterable, padvalue=None):
     """
